#!/bin/sh

source "@TSTPLIB@"

dep_check_programs dd mkfs.msdos modprobe
dep_check_programs ifconfig ping ssh sshpass

# This script helps to test the usb emulation of ethernet, mass storage and serial using g_ether, g_mass_storage and g_serial drivers respectively.

echo -e "Make sure usb gadget port cable is connected with the host machine.\n"

ETHIF=eth0
IPADDR="192.168.111.2"
USER=zuser
PASSWD=zuser

modulePath=/lib/modules/`uname -r`/kernel/drivers/usb/gadget/legacy

usage(){
        echo "`basename $0` -I '{ETHIF}' -A '{IPADDR}' -U '{USERNAME}' -P '{PASSWORD}' -T '{TESTCASE}'"
        echo ""
        echo "Where ETHIF         - Target Ethernet Interface"
        echo "      IPADDR        - Remote machine IP Address"
        echo "      USERNAME      - Remote machine Username"
        echo "      PASSWORD      - Remote machine Password corresponding to username provided"
        echo "      TESTCASE      - USB Gadget Module name which needs to be tested {g_ether/ g_serial/ g_mass_storage}"
        echo ""
        echo "Example: `basename $0` -I eth0 -A 192.168.111.2 -U zuser -P zpasswd -T g_ether"
	echo ""
        echo "      Help          - Show this message and quit"
        exit 1
}

checkConnectivity(){
	sysfsNetDir=/sys/class/net/$ETHIF

	if [ ! -d $sysfsNetDir ]; then
        	echo "Ethernet Interface $ETHIF Not Found"
        	exit 1
	else
			netIfState=`cat $sysfsNetDir/operstate`
        	if [ "$netIfState" = "up" ]; then
                	echo "Ethernet Interface $ETHIF is Up"
        	elif [ "$netIfState" = "down" ]; then
                	echo -e "Ethernet Interface $ETHIF is Down\nRunning udhcpc ..."
                	udhcpc -i $ETHIF -t 10 -n
			if [ $? -ne 0 ]; then
				echo "udhcpc failed to obtain an IP address"
				echo "Make sure working Ethernet cable is connected to the target"
				exit 1
			else
				echo "udhcpc successfully executed"
			fi
		fi
	fi

	echo "Checking Connectivity with $IPADDR"
	ping $IPADDR -c1 2>&1 >/dev/null
	if [ $? -ne 0 ]; then
		echo "No connectivity with $IPADDR"
		exit 1
	else
		echo "Ping test PASSED"
	fi
}

valid_ip(){
	if expr "$IPADDR" : '[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*\.[0-9][0-9]*$' >/dev/null; then
		for quad in 2 3; do
			if [ $(echo "$IPADDR" | cut -d. -f$quad) -gt 255 ]; then
				echo "fail ($IPADDR)"
				exit 1
				kill -s TERM $TOP_PID
			fi
		done
		if [ $(echo "$IPADDR" | cut -d. -f4) -gt 254 ]; then
			echo "fail ($IPADDR)"
			exit 1
			kill -s TERM $TOP_PID
		fi
		if [ $(echo "$IPADDR" | cut -d. -f1) -gt 254  ] && [ $(echo "$IPADDR" | cut -d. -f1) -lt 1  ]; then
			echo "fail ($IPADDR)"
			exit 1
			kill -s TERM $TOP_PID
		fi
		checkConnectivity
		echo "IP Address $IPADDR is Valid"
	else
		echo "IP Address $IPADDR is Not Valid"
		kill -s TERM $TOP_PID
		exit 1
	fi
}

check_package_installed(){
	pack=$1
	if ( which $pack > /dev/null 2>&1 ); then
		return 0
	else
		echo "$pack is not installed."
		exit 1
		kill -s TERM $TOP_PID
	fi
}

unloadModule(){
	module=$1
	if [ "$module" = "all" ]; then
		echo "Unload the existing usb gadget module if any"
		modprobe -r g_ether
		modprobe -r g_mass_storage
		modprobe -r g_serial
	else
		echo "Unload the module $module"
		modprobe -r $module
	fi
}

remoteSiteExecution(){
	if ( check_package_installed ssh && check_package_installed sshpass );then
		echo "Creating usb interface at remote site..."
		export SSHPASS=$PASSWD
		sshpass -e ssh -o "StrictHostKeyChecking no" $USER@$IPADDR 'bash -s' < ./bin/usb/usb_gadget_remote.sh $TESTCASE $PASSWD
		RESULT=$?
		unloadModule $TESTCASE
		exit $RESULT
	else
		unloadModule $TESTCASE
		exit 1
	fi
}

load_module(){
	module=$1
	echo "Load the module $module"
	if [ $module = "g_ether" ]; then
		modprobe $module || exit 1
		sleep 1
		ifconfig usb0 10.0.1.2 netmask 255.255.255.0
		remoteSiteExecution
	elif [ $module = "g_serial" ]; then
		modprobe $module || exit 1
		ls -l /dev/ttyGS0
		remoteSiteExecution
	elif [ $module = "g_mass_storage" ]; then
		dd if=/dev/zero of=/home/.testfile bs=1M count=100
		mkfs.msdos /home/.testfile
		modprobe $module file=/home/.testfile || exit 1
	else
		echo "Incorrect Module provided. Please select out of the provided testcases as module."
		echo "USB_Gadget: FAIL"
		exit 1
	fi
}

while getopts ":I:A:U:P:T:h" arg; do
	case "$arg" in
		I)
			ETHIF="${OPTARG}";;
		A)
			IPADDR="${OPTARG}";;
		U)
			USER="${OPTARG}";;
		P)
			PASSWD="${OPTARG}";;
		T)
			TESTCASE="${OPTARG}";;
		h)
			usage;;
		?)
			usage;;
	esac
done

if [ ! $TESTCASE ]; then
	echo "TESTCASE is a mandatory argument"
	usage
	exit 1
fi

main(){
	unloadModule all
	valid_ip
	load_module $TESTCASE
	if [ $? -ne 0 ]; then
		echo "USB Gadget: FAIL"
		exit 1
	fi
}

main
